/*
 * Proiect PM
 * Paul Vlase & Ghennadi Procopciuc
 * 335 CC
 * 15 / 5 / 2012
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

#include "opencv/cv.h"
#include "opencv/highgui.h"
#include "opencv/cxcore.h"

using namespace cv;

#define HEIGHT 	320
#define WIDTH  	240
#define WINDOW  40

#define DIRECTION   0


#define MY_IP		"10.10.10.2"
#define MY_PORT		60

#define STOP					0
#define FORWARD					1
#define BACKWARD				2
#define LEFT					3
#define RIGHT					4

int sockfd;
int n;
unsigned char direction;
unsigned char cmd[6];
unsigned char buffer[800];


void error(const char *msg)
{
    perror(msg);
    exit(0);
}


/**
 * Transforma din coordonatele cercului in parametrii comenzii ce va trimisa
 * microcontroller-ului. Modifica variabile globale.
 *
 * @param x
 *		coordonata x a cercului.
 * @param y
 *		coordonata y a cercului.
 *	@param radius
 *		raza cercului.
 */
void compute_params(float x, float y, float radius)
{
	direction 	= STOP;
	/* Asiguram o fereastra in centrul imaginii in care orice schimbare a
	   pozitiei mingii, nu implica si schimbarea pozitiei microcontroller-ului,
	   pentru un control mai fin al masinii. */

    if (x == -1) {
        return;
    }

	if(radius < 40){
		/* mergi in fata. */
		printf("Forward ");
		direction = FORWARD;

		if(x < (WIDTH / 2 - WINDOW / 2)) {
			/* mergi spre stanga. */
			printf("Left ");

			direction = LEFT;
		}

		if (x > (WIDTH / 2 + WINDOW / 2)) {
			/* mergi spre dreapta. */
			printf("Right ");

			direction = RIGHT;
		}
	} else {
		if(radius > 40 && radius < 60) {
			/* opreste-te de tot. */
			printf("Stop ");

			direction = STOP;
		} else {
			/* mergi in spate. */
			printf("Backward ");
			direction = BACKWARD;
		}
	}
	printf("\n");
}


void data_process(float x, float y, float radius)
{
	compute_params(x, y, radius);

	/* Impachetare comenzi. */
    n = write(sockfd,&direction,1);
    if (n < 0) {
        error("ERROR writing to socket");
    }

    printf("%u  ### ", direction);
    fflush(stdout);


	bzero(buffer, 256);

    n = read(sockfd, buffer, 255);
    if (n < 0) {
        error("ERROR reading from socket");
    }

    printf("%u\n", buffer[0]);
    fflush(stdout);
}


/**
 * Initializare socket pentru comunicatia cu microcontroller.
 */
void eth_init()
{
    int portno, i, n;
    struct sockaddr_in serv_addr;
    struct hostent *server;
    int val;
    unsigned direction, right_speed, left_speed;

    portno = MY_PORT;
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
        error("ERROR opening socket");

    server = gethostbyname(MY_IP);
    if (server == NULL) {
        fprintf(stderr,"ERROR, no such host\n");
        exit(0);
    }

    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr,
            (char *)&serv_addr.sin_addr.s_addr,
            server->h_length);
    serv_addr.sin_port = htons(portno);

    if (connect(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0) {
        error("ERROR connecting");
    }
}


int main (int argc, char **argv)
{
	int i;
	int min_red, min_green, min_blue;
	int max_red, max_green, max_blue;
	float* p;
	CvCapture 	*capture;
    CvScalar 	hsv_min;
    CvScalar 	hsv_max;
	CvSize 		size;
	IplImage*   hsv_frame;
	IplImage*   thresholded;

	eth_init();

    capture = cvCaptureFromFile("http://10.38.152.171:8080/videofeed");
    if( !capture )
    {
            fprintf( stderr, "ERROR: capture is NULL \n" );
            getchar();
            return -1;
    }

	min_red 	= 0;
	max_red 	= 255;
	min_green 	= 81;
	max_green 	= 255;
	min_blue 	= 86;
	max_blue 	= 255;

    // Detecteaza o minge. */
    size = cvSize(HEIGHT, WIDTH);
    thresholded	= cvCreateImage(size, IPL_DEPTH_8U, 1);
    hsv_frame	= cvCreateImage(size, IPL_DEPTH_8U, 3);

	// O fereastra noua. */
	cvNamedWindow("Color Tune",CV_WINDOW_NORMAL);

	cvCreateTrackbar( "Min red","Color Tune", &min_red, 255, 0 );
	cvCreateTrackbar ("Max red","Color Tune", &max_red, 255, 0);
	cvCreateTrackbar( "Min green","Color Tune", &min_green, 255, 0 );
	cvCreateTrackbar( "Max green","Color Tune", &max_green, 255, 0 );
	cvCreateTrackbar( "Min blue","Color Tune", &min_blue, 255, 0 );
	cvCreateTrackbar( "Max Blue","Color Tune", &max_blue, 255, 0 );

	// Obtine un cadru si il afiseaza. */
	while(1)
	{
		/* Obtine cate o imagine (cadru) de la stream-ul video. */
        IplImage* frame = cvQueryFrame( capture );
        if(!frame)
        {
                fprintf( stderr, "ERROR: frame is null...\n" );
                getchar();
                break;
        }

		hsv_min = cvScalar(min_red, min_green, min_blue, 0);
		hsv_max = cvScalar(max_red, max_green, max_blue, 0);

		/* Converteste spatiul de culori in HSV, deoarece este mai usor de
		   aplicat filtrul de culori in spatiul HSV. */
        cvCvtColor(frame, hsv_frame, CV_BGR2HSV);
		/* Filtreaza culorile care nu se afla in intervalul dorit. */
        cvInRangeS(hsv_frame, hsv_min, hsv_max, thresholded);

        /* Memorie pentru cercurile hough. */
        CvMemStorage* storage = cvCreateMemStorage(0);

		/* Detectorul hough functioneaza mult mai bine daca este aplicat un
		   smooth peste imagine. */
        cvSmooth(thresholded, thresholded, CV_GAUSSIAN, 9, 9);

        CvSeq* circles = cvHoughCircles(thresholded, storage, CV_HOUGH_GRADIENT,
        		2, INT_MAX, 100, 40, 1, 400);

        for (i = 0; i < circles->total; i++)
        {
            p = (float*) cvGetSeqElem(circles, i);

            printf("Ball! x=%f y=%f r=%f\n\r", p[0], p[1], p[2]);

            cvCircle(frame, cvPoint(cvRound(p[0]), cvRound(p[1])), 3,
            		CV_RGB(0,255,0), -1, 8, 0 );
            cvCircle(frame, cvPoint(cvRound(p[0]), cvRound(p[1])),
            		cvRound(p[2]), CV_RGB(255,0,0), 3, 8, 0);

			data_process(p[0], p[1], p[2]);
        }
        if (circles->total == 0 ) {
            data_process(-1, -1, -1);
        }

		/* Stream-ul initial, cu conturul mingii detectate. */
        cvShowImage( "Camera", frame );
        /* Stream-ul initial in spatiul de culori HSV. */
        cvShowImage("HSV", hsv_frame);
        /* Stream-ul dupa filtrarea culorilor. */
        cvShowImage("After Color Filtering", thresholded );
        cvReleaseMemStorage(&storage);

        /* Tasta ESC a fost apasata (Key=0x10001B). */
        if( (cvWaitKey(10) & 255) == 27 ) break;
	}

	/* CLEAN UP */
    close(sockfd);
	cvDestroyAllWindows();
}
