/* 
 * Proiect PM
 * Paul Vlase & Ghennadi Procopciuc 
 * 335 CC
 * 15 / 5 / 2012
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>
#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "timeout.h"
#include "avr_compat.h"
#include "net.h"

static uint8_t mymac[6]     =   {0x54, 0x55, 0x58, 0x10, 0x00, 0x24};
static uint8_t myip[4]      =   {10, 10, 10, 2};
#define MY_IP_STR               "10.10.10.2"

#define LEFT_MOTOR_SPEED		OCR0
#define RIGHT_MOTOR_SPEED		OCR2

#define	LEFT_MOTOR_BACKWARD		PC0
#define LEFT_MOTOR_FORWARD		PC1

#define	RIGHT_MOTOR_BACKWARD	PC3
#define RIGHT_MOTOR_FORWARD		PC4

#define STOP					0
#define FORWARD					1
#define BACKWARD				2
#define LEFT					3
#define RIGHT					4

// IP
uint8_t yip[4];

static char baseurl[]       =   "http://10.10.10.2/";

static uint16_t mywwwport   =   60;

#define BUFFER_SIZE             800

unsigned char buf[BUFFER_SIZE + 1];

unsigned char data[4];
unsigned char direction;

void pwm_init(){
	
	// Motor 1
	{
		DDRB |= _BV(PB3);
		TCCR0 |= _BV(WGM00) | _BV(COM01) | _BV(CS02);
	}
	
	// Motor 2
	{
		DDRD |= _BV(PD7);
		TCCR2 |= _BV(WGM20) | _BV(COM21) | _BV(CS22) | _BV(CS21);
	}
}

void set_direction(int direction){
	if(direction == STOP){
		PORTC &= ~_BV(LEFT_MOTOR_BACKWARD);
		PORTC &= ~_BV(LEFT_MOTOR_FORWARD);
		
		PORTC &= ~_BV(RIGHT_MOTOR_BACKWARD);
		PORTC &= ~_BV(RIGHT_MOTOR_FORWARD);
		
		LEFT_MOTOR_SPEED = 0;
		RIGHT_MOTOR_SPEED = 0;
		
		return;
	}
	
	if(direction == FORWARD){
		PORTC |= _BV(LEFT_MOTOR_FORWARD) | _BV(RIGHT_MOTOR_FORWARD);
		return;
	}
	
	if(direction == BACKWARD){
		PORTC |= _BV(LEFT_MOTOR_BACKWARD) | _BV(RIGHT_MOTOR_BACKWARD);
		return;
	}
}

int main(void) {
        uint16_t plen;
        uint16_t dat_p;
        uint8_t i = 0;
        int8_t cmd;
		
		unsigned char *data_p;
		
		// Input enable
		DDRC |= 0x1b;
		
		pwm_init();
		
		
        delay_ms(10);
        sei();

        DDRD |= (1<<DDD4);
        DDRA |= (1<<DDA0) | (1<<DDA1) | (1<<DDA2) | (1<<DDA3) |
                (1<<DDA4) | (1<<DDA5) | (1<<DDA6) | (1<<DDA7);

        // Set output to gnd, reset the ethernet chip.
        PORTD &= ~(1<<PD4);
        delay_ms(10);

        // Set output to Vcc, reset inactive.
        PORTD |= (1<<PD4);
        delay_ms(200);

        // Initialize enc28j60.
        enc28j60Init(mymac);
        delay_ms(20);

        // Enable PB1, LED as output.
        DDRB |= (1<<DDB0);

        // Set output to Vcc, LED off.
        PORTB |= (1<<PB1);

        enc28j60PhyWrite(PHLCON, 0x476);
        delay_ms(20);

        // Set output to GND, red LED on.
        PORTB &= ~(1<<PB0);
        i = 1;

        // Init the ethernet/ip layer.
        init_ip_arp_udp_tcp(mymac, myip, mywwwport);

        while (1) {
                // Get the next new packet.
                plen = enc28j60PacketReceive(BUFFER_SIZE, buf);

                // plen will be different than zero if there is a valid
                // packet (without crc error).
                if (plen == 0) {
                        continue;
                }

                // Arp is broadcast if unknown but a host may also
                // verify the mac address by sending it to
                // a unicast address.
                if (eth_type_is_arp_and_my_ip(buf, plen)) {
                        make_arp_answer_from_request(buf);
                        continue;
                }

                // Check if ip packets are for us.
                if (eth_type_is_ip_and_my_ip(buf,plen) == 0) {
                        continue;
                }

                if (buf[IP_PROTO_P] == IP_PROTO_ICMP_V && buf[ICMP_TYPE_P] == ICMP_TYPE_ECHOREQUEST_V) {
                        // A ping packet, let's send pong.
                        make_echo_reply_from_request(buf,plen);
                        continue;
                }

                // TCP port www start, compare only the lower byte.
                if (buf[IP_PROTO_P] == IP_PROTO_TCP_V && buf[TCP_DST_PORT_H_P] == 0 
				    && buf[TCP_DST_PORT_L_P] == mywwwport) {
                        // Read source IP.
                        yip[0] = buf[26];
                        yip[1] = buf[27];
                        yip[2] = buf[28];
                        yip[3] = buf[29];

                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V) {
                                make_tcp_synack_from_syn(buf);
                                // make_tcp_synack_from_syn does already send the syn,ack.
                                continue;
                        }

                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V) {
                                init_len_info(buf);

                                // We can possibly have no data, just ack.
                                dat_p = get_tcp_data_pointer();
                                if (dat_p == 0) {
                                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V) {
                                                // Finack, answer with ack.
                                                make_tcp_ack_from_any(buf);
                                        }

                                        // Just an ack with no data, wait for next packet.
                                        continue;
                                }
								
								//PORTA ^= 0xFF;
								
								direction = (buf[dat_p] << 4) | buf[dat_p + 1];
								
								switch (direction)
								{
								case STOP: {
									PORTC &= ~_BV(LEFT_MOTOR_BACKWARD);
									PORTC &= ~_BV(LEFT_MOTOR_FORWARD);
		
									PORTC &= ~_BV(RIGHT_MOTOR_BACKWARD);
									PORTC &= ~_BV(RIGHT_MOTOR_FORWARD);
		
									LEFT_MOTOR_SPEED	= 0;
									RIGHT_MOTOR_SPEED	= 0;
								}
									break;
								
								case FORWARD: {
									PORTC |= _BV(LEFT_MOTOR_FORWARD) | _BV(RIGHT_MOTOR_FORWARD);
									PORTC &= ~_BV(LEFT_MOTOR_BACKWARD);
									PORTC &= ~_BV(RIGHT_MOTOR_BACKWARD);
									
									LEFT_MOTOR_SPEED	= 255;
									RIGHT_MOTOR_SPEED	= 255;
								}
									break;
								
								case BACKWARD: {
									PORTC |= _BV(LEFT_MOTOR_BACKWARD) | _BV(RIGHT_MOTOR_BACKWARD);									
									PORTC &= ~_BV(LEFT_MOTOR_FORWARD);
									PORTC &= ~_BV(RIGHT_MOTOR_FORWARD);
									
									LEFT_MOTOR_SPEED	= 255;
									RIGHT_MOTOR_SPEED	= 255;
								}
									break;
									
								case LEFT: {
									PORTC |= _BV(LEFT_MOTOR_BACKWARD) | _BV(RIGHT_MOTOR_FORWARD);
									PORTC &= ~_BV(LEFT_MOTOR_FORWARD);
									PORTC &= ~_BV(RIGHT_MOTOR_BACKWARD);

									LEFT_MOTOR_SPEED	= 255;
									RIGHT_MOTOR_SPEED	= 255;
								}
									break;
									
								case RIGHT: {
									PORTC |= _BV(LEFT_MOTOR_FORWARD) | _BV(RIGHT_MOTOR_BACKWARD);
									PORTC &= ~_BV(LEFT_MOTOR_BACKWARD);
									PORTC &= ~_BV(RIGHT_MOTOR_FORWARD);
					
									LEFT_MOTOR_SPEED	= 255;
									RIGHT_MOTOR_SPEED	= 255;		
								}
									break;
								};
								
								buf[TCP_CHECKSUM_L_P+3]		= direction >> 4;
								buf[TCP_CHECKSUM_L_P+3+1]	= direction & 0x0F;
								
								plen = 2;
								
                                make_tcp_ack_from_any(buf);
                                make_tcp_ack_with_data(buf, plen);
								
                                continue;
                        }
                }
        }

        return 0;
}
